/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "filedescriptor.h"

#include <errno.h>

namespace Conexus
  {

  FileDescriptor::FileDescriptor(int fd, bool close_on_destruction): Endpoint(close_on_destruction), m_fd(fd)
  {}


  FileDescriptor::~FileDescriptor()
  {
    if (m_close_on_destruction && is_open() )
      close();
  }

  int FileDescriptor::fd() throw ()
  {
    return m_fd;
  }

  void FileDescriptor::close(bool force) throw (close_error)
  {
    if ( is_closed() && !force )
      return;
    
    int i = ::close(m_fd);
    if (i == -1)
      throw_close_error(errno);

    set_state_closed();
  }

  void FileDescriptor::read_thread_main()
  {
    int n;
    Data d;
    fd_set readfd;
    struct timeval tv;

    if (m_fd < 0)
      throw;

    while ( ! m_terminate /*true*/ )
      {
        FD_ZERO(&readfd);
        FD_SET(m_fd, &readfd);

        if (m_timeval.tv_usec <= 0)
          n = select(m_fd+1, &readfd, NULL, NULL, NULL);
        else
        {
          tv = m_timeval;
          n = select(m_fd+1, &readfd, NULL, NULL, &tv);
        }

        if (FD_ISSET(m_fd, &readfd))
          {
            try
              {
                d = read();
                if (d.size() > 0)
                  queue_received_data(d);
              }
            catch (error::read::disconnected)
              {
                break; // break out of the while loop
              }
          }
      }
      m_running[READ] = false;
      this->read_thread_cleanup();
  }

}
