/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef IPV4IPV4_TCPSERVER_H
#define IPV4IPV4_TCPSERVER_H

#include <conexus/ipv4_address.h>
#include <conexus/endpointserver.h>
#include <conexus/socket.h>

/**
 * \page manual_ipv4_tcp_server IPv4 TCPServer Endpoint
 * 
 * For the corresponding clients, see \ref manual_ipv4_tcp
 * 
 * \section example_ipv4_tcpserver_server IPv4 TCP Server Example
 * \include ipv4_tcp_server.cpp
 * 
 * \section example_ipv4_tcpserver_echo_server IPv4 TCP Echo Server Example
 * \include ipv4_tcp_echo_server.cpp
 */
 
namespace Conexus
  {

  namespace IPv4
    {

    /**
     * The TCPServer generates TCP endpoints when a connection request is made.
     * 
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     */
    class TCPServer: public Conexus::EndpointServer
      {
      public:
        typedef boost::shared_ptr<TCPServer> pointer;
        
        TCPServer(uint16_t port=0);
        
        static TCPServer::pointer create(uint16_t port=0);

        virtual ~TCPServer();

        virtual Address& local_interface();

        virtual void set_local_interface(Address addr);

        virtual void start();
        virtual void start(bool use_dispatcher);
        virtual void stop();

        virtual const std::string& object_type() { static std::string s("Conexus::IPv4::TCPServer"); return s; }

        protected:
        Conexus::Socket m_tcp_listener;
        Address m_local_interface;


        virtual void service_thread_main();


      };

  } // namespace IPv4

} // namespace Conexus

#endif
